#include <MsTimer2.h>

///////////////////////////////////////////////////////
//////////////////////VARIABLES////////////////////////
///////////////////////////////////////////////////////

// Encoder and interrupt pins and variables
int encoderPinB[2] = {25,22}; // Encoder B pin
long encoderTicks[2];
int odoLPin = 4; // interrupt 4 for digital Pin 19 
int odoRPin = 5; // interrupt 5 for digital Pin 18

// Constants
#define COEF_Dist 11.0184191 // 1 cm -> 11.0184191 encoder ticks
#define COEF_ROT 4.352083   //  1°    -> 4.35 encoder ticks 
#define PI 3.14159265
#define CONV_RD PI/(180*COEF_ROT) // angle conversion from ticks to radian

//  Global variables
long dist,last_dist ; // dist : distance covered at instant tn ; last_dist : distance covered at instant tn-1
long orient, last_orient, orient_init; // orient : current angle at instant tn; last_orient : last angle at instant tn-1
float x, y; // (x,y) coordinates in ticks
int x_cm, y_cm; // (x,y) coordinates in cm

// Motor Driving variables
 #define TOPSPEED 200
 #define FORWARD_SPEED 150
 #define TURN_SPEED 60
 // Left motor 
 int dir1PinA = 13;
 int dir2PinA = 12;
 int speedPinA = 10;
 // Right motor 
 int dir1PinB = 11;
 int dir2PinB = 8;
 int speedPinB = 9;
 int oam_speed[2]={0}; //oam_speed[0] = left wheel ; oam_speed[1] = right wheel

 // Dead reckoning variables
 int delta_d, delta_orient; // difference in dist ; difference in angle
 long orient_mean; // mean of current and last angle
 float orient_mean_radian, delta_orient_radian;  // radian converted 
 float K=1, dx,dy; // differential x ; differential y
 
///////////////////////////////////////////////////////
//////////////////////FUNCTIONS////////////////////////
///////////////////////////////////////////////////////

// Initial Conditions
void init_dead_reckoning (float x0, float y0, long teta0)
{ 
  orient_init = teta0 ;
  last_orient = teta0 ;
  x = x0 ;
  y = y0 ;
  dist = 0 ;
  last_dist=0 ;
}

// Update function executed every 5 ms
void update()
{
  dead_reckoning () ; // Coordinates calculation

  while(orient_mean_radian<0){orient_mean_radian+=2*PI;} // Keeping angle in a [0, 2*pi] interval
  while(orient_mean_radian>2*PI){orient_mean_radian-=2*PI;}
}

// Dead reckoning
void dead_reckoning (void)
{ 
  dist = ((encoderTicks[1]+encoderTicks[0])/2); // distance covered at instant tn
  orient = orient_init +(encoderTicks[1]-encoderTicks[0]); 
  delta_d = dist - last_dist; 
  delta_orient = orient - last_orient; 
  orient_mean = (orient +last_orient) / 2;  
  delta_orient_radian = CONV_RD*delta_orient; 
  orient_mean_radian = CONV_RD*orient_mean; 
  
  dx=K*delta_d*cos(orient_mean_radian);
  dy=K*delta_d*sin(orient_mean_radian);
  x= x + dx; // Values in ticks
  y= y + dy;
  x_cm=(int)(x/COEF_Dist); // This calculus can be done outside the dead_reckoning function
  y_cm=(int)(y/COEF_Dist); 
  last_orient = orient ; // Refreshing angle value
  last_dist = dist ; // Refreshing distance value
}
// Left encoder Ticks 
void incL(){   
  if (digitalRead(encoderPinB[0]) == HIGH){  
     encoderTicks[0]++; 
        
  } else {  
     encoderTicks[0]--; 
  }  
}  

// Left encoder Ticks   
void incR(){  
  if (digitalRead(encoderPinB[1]) == LOW){  
     encoderTicks[1]++; 
          
  } else {  
     encoderTicks[1]--; 
       
  }  
} 

void  init_encoders(){
pinMode(encoderPinB[0], INPUT); // Encoder pin B INPUT
pinMode(encoderPinB[1], INPUT); // Encoder pin B INPUT
}

void init_motors(){
  pinMode(dir1PinA, OUTPUT);
  pinMode(dir2PinA, OUTPUT);
  pinMode(speedPinA, OUTPUT);
  pinMode(dir1PinB, OUTPUT);
  pinMode(dir2PinB, OUTPUT);
  pinMode(speedPinB, OUTPUT);
}

// Setup and main loop

void setup (){
  init_dead_reckoning(0,0,0);
  // Serial communication
  Serial.begin(38400);
  init_motors(); // Motor Pin intialization
  init_encoders(); //Encoders Pin initialization
 
  // attach the odometer interrupt
  attachInterrupt(odoLPin, incL, RISING); //18
  attachInterrupt(odoRPin, incR, RISING); //19
  // Timer2 to execute update function
  MsTimer2::set(5, update); // 5ms period
  MsTimer2::start();
}

void loop(){
  
  //  Execute a full revolution then stop
  while(orient_mean_radian*180/PI < 360){
  analogWrite(speedPinA,TOPSPEED);
  analogWrite(speedPinB,TOPSPEED);
  digitalWrite(dir1PinA, LOW);
  digitalWrite(dir2PinA, HIGH);
  digitalWrite(dir1PinB, HIGH);
  digitalWrite(dir2PinB, LOW);
  }

  analogWrite(speedPinA,0);
  analogWrite(speedPinB,0);

   /* Serial.print(encoderTicks[0]);
    Serial.print("|");
    Serial.print(encoderTicks[1]);
    Serial.print("\tx = ");
    Serial.print(x_cm);
    Serial.print("| y = ");
    Serial.print(y_cm);
    Serial.print("\tTheta = ");
    Serial.println(orient_mean_radian*180/PI);*/
}
